{*********************************************************}
{* AAFltStk                                              *}
{* Copyright (c) Julian M Bucknall 1998-1999             *}
{* All rights reserved.                                  *}
{*********************************************************}
{* Stack of floating point values (doubles)              *}
{*********************************************************}

{Note: this unit is released as freeware. In other words, you are free
       to use this unit in your own applications, however I retain all
       copyright to the code. JMB}

unit AAFltStk;

interface

uses
  SysUtils;

type
  TaaFloatStack = class
    private
      FStack : PChar;
      FSP    : integer;
      FSize  : integer;
    protected
    public
      constructor Create;
        {-create the double type stack}
      destructor Destroy; override;
        {-destroy the double type stack; releasing all memory}

      procedure Clear;
        {-remove all doubles from stack}
      function Count : integer;
        {-count of doubles on the stack}
      function Examine : double;
        {-return the top double value from the stack; don't pop it}
      function IsEmpty : boolean;
        {-is the stack empty?}
      function Pop : double;
        {-pop the top double value from the stack; return it}
      procedure Push(aValue : double);
        {-push the given double onto the stack}
    end;

implementation

type
  PDouble = ^double;

{===TaaFloatStack====================================================}
constructor TaaFloatStack.Create;
begin
  inherited Create;
  FSize := 1024;
  GetMem(FStack, FSize);
  FSP := -1;
end;
{--------}
destructor TaaFloatStack.Destroy;
begin
  if (FStack <> nil) then
    FreeMem(FStack, FSize);
  inherited Destroy;
end;
{--------}
procedure TaaFloatStack.Clear;
begin
  FSP := -1;
end;
{--------}
function TaaFloatStack.Count : integer;
begin
  Result := succ(FSP);
end;
{--------}
function TaaFloatStack.Examine : double;
begin
  {check for the obvious mistake}
  if (FSP = -1) then
    raise Exception.Create('TaaFloatStack.Examine: the stack is empty');
  {return the current character}
  Result := PDouble(@FStack[FSP])^;
end;
{--------}
function TaaFloatStack.IsEmpty : boolean;
begin
  Result := (FSP = -1);
end;
{--------}
function TaaFloatStack.Pop : double;
begin
  {check for the obvious mistake}
  if (FSP = -1) then
    raise Exception.Create('TaaFloatStack.Pop: the stack is empty');
  {return the current character}
  Result := PDouble(@FStack[FSP])^;
  {decrement the stack pointer}
  dec(FSP, sizeof(double));
end;
{--------}
procedure TaaFloatStack.Push(aValue : double);
var
  NewSize  : integer;
  NewStack : PChar;
begin
  {increment the stack pointer}
  inc(FSP, sizeof(double));
  {if we've run out of space, reallocate the stack buffer}
  if (FSP >= FSize) then begin
    NewSize := FSize + 256;
    GetMem(NewStack, NewSize);
    Move(FStack^, NewStack^, FSize);
    FreeMem(FStack, FSize);
    FStack := NewStack;
    FSize := NewSize;
  end;
  {store the pushed character}
  PDouble(@FStack[FSP])^ := aValue;
end;
{====================================================================}

end.
